@@ -15,7 +15,7 @@ var secret = "";
 var session = null;		// Selected session
 var handle = null;		// Selected handle
 
-var plugins = [];
+var plugins = [], pluginsIndex = [];
 var settings = {};
 
 var currentHandle = null;
@@ -79,6 +79,8 @@ function randomString(len) {
 
 // Server info
 function updateServerInfo() {
+	plugins = [];
+	pluginsIndex = [];
 	$.ajax({
 		type: 'GET',
 		url: server + "/info",
@@ -106,7 +108,25 @@ function updateServerInfo() {
 			delete json.plugins;
 			delete json.transports;
 			delete json.events;
+			$('#server-details').empty();
 			for(var k in json) {
+				if(k === "dependencies") {
+					$('#server-deps').html(
+						'<tr>' +
+						'	<th>Library</th>' +
+						'	<th>Version</th>' +
+						'</tr>'
+					);
+					for(var ln in json[k]) {
+						$('#server-deps').append(
+							'<tr>' +
+							'	<td>' + ln + '</td>' +
+							'	<td>' + json[k][ln] + '</td>' +
+							'</tr>'
+						);
+					}
+					continue;
+				}
 				var v = json[k];
 				$('#server-details').append(
 					'<tr>' +
@@ -114,6 +134,15 @@ function updateServerInfo() {
 					'	<td>' + v + '</td>' +
 					'</tr>');
 			}
+			$('#server-plugins').html(
+				'<tr>' +
+				'	<th>Name</th>' +
+				'	<th>Author</th>' +
+				'	<th>Description</th>' +
+				'	<th>Version</th>' +
+				'</tr>'
+			);
+			$('#plugins-list').empty();
 			for(var p in pluginsJson) {
 				plugins.push(p);
 				var v = pluginsJson[p];
@@ -124,7 +153,28 @@ function updateServerInfo() {
 					'	<td>' + v.description + '</td>' +
 					'	<td>' + v.version_string + '</td>' +
 					'</tr>');
+				pluginsIndex.push(p);
+				$('#plugins-list').append(
+					'<a id="plugin-'+(pluginsIndex.length-1)+'" href="#" class="list-group-item">'+p+'</a>'
+				);
+				$('#plugin-'+(pluginsIndex.length-1)).click(function(event) {
+					event.preventDefault();
+					var pi = parseInt($(this).attr('id').split('plugin-')[1]);
+					var plugin = pluginsIndex[pi];
+					console.log("Selected plugin:", plugin);
+					$('#plugins-list a').removeClass('active');
+					$('#plugin-'+pi).addClass('active');
+					resetPluginRequest();
+				});
 			}
+			$('#server-transports').html(
+				'<tr>' +
+				'	<th>Name</th>' +
+				'	<th>Author</th>' +
+				'	<th>Description</th>' +
+				'	<th>Version</th>' +
+				'</tr>'
+			);
 			for(var t in transportsJson) {
 				var v = transportsJson[t];
 				$('#server-transports').append(
@@ -135,6 +185,14 @@ function updateServerInfo() {
 					'	<td>' + v.version_string + '</td>' +
 					'</tr>');
 			}
+			$('#server-handlers').html(
+				'<tr>' +
+				'	<th>Name</th>' +
+				'	<th>Author</th>' +
+				'	<th>Description</th>' +
+				'	<th>Version</th>' +
+				'</tr>'
+			);
 			for(var e in eventsJson) {
 				var v = eventsJson[e];
 				$('#server-handlers').append(
@@ -313,6 +371,22 @@ function updateSettings() {
 							setNoMediaTimer(result);
 						});
 					});
+				} else if(k === 'slowlink_threshold') {
+					$('#'+k).append('<button id="' + k + '_button" type="button" class="btn btn-xs btn-primary">Edit slowlink-threshold value</button>');
+					$('#'+k + "_button").click(function() {
+						bootbox.prompt("Set the new desired slowlink-threshold value (in lost packets per seconds, currently " + settings["slowlink_threshold"] + ")", function(result) {
+							if(isNaN(result)) {
+								bootbox.alert("Invalid slowlink-threshold timer (should be a positive integer)");
+								return;
+							}
+							result = parseInt(result);
+							if(result < 0) {
+								bootbox.alert("Invalid slowlink-threshold timer (should be a positive integer)");
+								return;
+							}
+							setSlowlinkThreshold(result);
+						});
+					});
 				} else if(k === 'locking_debug') {
 					$('#'+k).append('<button id="' + k + '_button" type="button" class="btn btn-xs"></button>');
 					$('#'+k + "_button")
@@ -446,6 +520,11 @@ function setNoMediaTimer(timer) {
 	sendSettingsRequest(request);
 }
 
+function setSlowlinkThreshold(packets) {
+	var request = { "janus": "set_slowlink_threshold", "slowlink_threshold": packets, "transaction": randomString(12), "admin_secret": secret };
+	sendSettingsRequest(request);
+}
+
 function sendSettingsRequest(request) {
 	console.log(request);
 	$.ajax({
@@ -486,6 +565,130 @@ function sendSettingsRequest(request) {
 	});
 }
 
+// Plugins
+function resetPluginRequest() {
+	$('#plugin-request').empty().append(
+		'<tr style="background: #f9f9f9;">' +
+		'	<th width="25%">Name</th>' +
+		'	<th width="25%">Value</th>' +
+		'	<th width="25%">Type</th>' +
+		'	<th></th>' +
+		'</tr>' +
+		'<tr>' +
+		'	<td><i id="addattr" class="fa fa-plus-circle" style="cursor: pointer;"></i></td>' +
+		'	<td></td>' +
+		'	<td></td>' +
+		'	<td><button id="sendmsg" type="button" class="btn btn-xs btn-success pull-right">Send message</button></td>' +
+		'</tr>');
+	$('#addattr').click(addPluginMessageAttribute).click();
+	$('#sendmsg').click(function() {
+		var message = {};
+		var num = $('.pm-property').length;
+		for(var i=0; i<num; i++) {
+			var name = $('#attrname'+i).val();
+			if(name === '') {
+				bootbox.alert("Missing name in attribute #" + (i+1));
+				return;
+			}
+			if(message[name] !== null && message[name] !== undefined) {
+				bootbox.alert("Duplicate attribute '" + name + "'");
+				return;
+			}
+			var value = $('#attrvalue'+i).val();
+			if(value === '') {
+				bootbox.alert("Missing value in attribute #" + (i+1));
+				return;
+			}
+			var type = $('#attrtype'+i).val();
+			if(type === "number") {
+				value = parseInt(value);
+				if(isNaN(value)) {
+					bootbox.alert("Invalid value in attribute #" + (i+1) + " (expecting a number)");
+					return;
+				}
+			} else if(type === "boolean") {
+				if(value.toLowerCase() === "true") {
+					value = true;
+				} else if(value.toLowerCase() === "false") {
+					value = false;
+				} else {
+					bootbox.alert("Invalid value in attribute #" + (i+1) + " (expecting a boolean)");
+					return;
+				}
+			}
+			console.log("Type:", type);
+			message[name] = value;
+		}
+		sendPluginMessage($('#plugins-list .active').text(), message);
+	});
+	$('#plugin-message').removeClass('hide');
+}
+
+function addPluginMessageAttribute() {
+	var num = $('.pm-property').length;
+	$('#addattr').parent().parent().before(
+		'<tr>' +
+		'	<td><input type="text" id="attrname' + num + '" placeholder="Attribute name" onkeypress="return checkEnter(this, event);" style="width: 100%;" class="pm-property form-control input-sm"></td>' +
+		'	<td><input type="text" id="attrvalue' + num + '" placeholder="Attribute value" onkeypress="return checkEnter(this, event);" style="width: 100%;" class="form-control input-sm"></td>' +
+		'	<td>' +
+		'		<select id="attrtype' + num + '" class="form-control input-sm">' +
+		'			<option>string</option>' +
+		'			<option>number</option>' +
+		'			<option>boolean</option>' +
+		'		</select>' +
+		'	</td>' +
+		'	<td></td>' +
+		'</tr>'
+	);
+}
+
+function sendPluginMessage(plugin, message) {
+	console.log("Sending message to " + plugin + ":", message);
+	var request = {
+		janus: "message_plugin",
+		transaction: randomString(12),
+		admin_secret: secret,
+		plugin: plugin,
+		request: message
+	};
+	$.ajax({
+		type: 'POST',
+		url: server,
+		cache: false,
+		contentType: "application/json",
+		data: JSON.stringify(request),
+		success: function(json) {
+			if(json["janus"] !== "success") {
+				console.log("Ooops: " + json["error"].code + " " + json["error"].reason);	// FIXME
+				var authenticate = (json["error"].code === 403);
+				if(!authenticate || (authenticate && !prompting && !alerted)) {
+					if(authenticate)
+						alerted = true;
+					bootbox.alert(json["error"].reason, function() {
+						if(authenticate) {
+							promptAccessDetails();
+							alerted = false;
+						}
+					});
+				}
+			}
+			$('#plugin-response').text(JSON.stringify(json, null, 4));
+		},
+		error: function(XMLHttpRequest, textStatus, errorThrown) {
+			console.log(textStatus + ": " + errorThrown);	// FIXME
+			if(!prompting && !alerted) {
+				alerted = true;
+				bootbox.alert("Couldn't contact the backend: is Janus down, or is the Admin/Monitor interface disabled?", function() {
+					promptAccessDetails();
+					alerted = false;
+				});
+			}
+		},
+		dataType: "json"
+	});
+}
+
+
 // Handles
 function updateSessions() {
 	$('#update-sessions').unbind('click').addClass('fa-spin');
@@ -1224,6 +1427,8 @@ function checkEnter(field, event) {
 	if(theCode == 13) {
 		if(field.id == 'token')
 			$('#addtoken').click();
+		else if(field.id.indexOf('attr') !== -1)
+			$('#sendmsg').click();
 		return false;
 	} else {
 		return true;
