@@ -309,7 +309,7 @@ function updateStreamsList() {
 				return false;
 
 			});
-			$('#watch').removeAttr('disabled').click(startStream);
+			$('#watch').removeAttr('disabled').unbind('click').click(startStream);
 		}
 	}});
 }
@@ -342,7 +342,7 @@ function stopStream() {
 	streaming.hangup();
 	$('#streamset').removeAttr('disabled');
 	$('#streamslist').removeAttr('disabled');
-	$('#watch').html("Watch or Listen").removeAttr('disabled').click(startStream);
+	$('#watch').html("Watch or Listen").removeAttr('disabled').unbind('click').click(startStream);
 	$('#status').empty().hide();
 	$('#bitrate').attr('disabled', true);
 	$('#bitrateset').html('Bandwidth<span class="caret"></span>');
