@@ -58,7 +58,7 @@
  * \verbatim
 [<unique room ID>]
 description = This is my awesome room
-is_private = true|false (whether this room should be in the public list, default=yes)
+is_private = true|false (whether this room should be in the public list, default=true)
 secret = <optional password needed for manipulating (e.g. destroying) the room>
 pin = <optional password needed for joining the room>
 post = <optional backend to contact via HTTP post for all incoming messages>
@@ -163,7 +163,7 @@ post = <optional backend to contact via HTTP post for all incoming messages>
 	"new_secret" : "<new password required to edit/destroy the room; optional>",
 	"new_pin" : "<new password required to join the room; optional>",
 	"new_is_private" : <true|false, whether the room should appear in a list request; optional>,
-	"permanent" : <true|false, whether the room should be also removed from the config file; default false>
+	"permanent" : <true|false, whether the room should be also removed from the config file; default=false>
 }
 \endverbatim
  *
@@ -186,7 +186,7 @@ post = <optional backend to contact via HTTP post for all incoming messages>
 	"textroom" : "destroy",
 	"room" : <unique numeric ID of the room to destroy; mandatory>,
 	"secret" : "<room secret; mandatory if configured>",
-	"permanent" : <true|false, whether the room should be also removed from the config file; default false>
+	"permanent" : <true|false, whether the room should be also removed from the config file; default=false>
 }
 \endverbatim
  *
@@ -470,10 +470,11 @@ const char *janus_textroom_get_author(void);
 const char *janus_textroom_get_package(void);
 void janus_textroom_create_session(janus_plugin_session *handle, int *error);
 struct janus_plugin_result *janus_textroom_handle_message(janus_plugin_session *handle, char *transaction, json_t *message, json_t *jsep);
+json_t *janus_textroom_handle_admin_message(json_t *message);
 void janus_textroom_setup_media(janus_plugin_session *handle);
 void janus_textroom_incoming_rtp(janus_plugin_session *handle, int video, char *buf, int len);
 void janus_textroom_incoming_rtcp(janus_plugin_session *handle, int video, char *buf, int len);
-void janus_textroom_incoming_data(janus_plugin_session *handle, char *buf, int len);
+void janus_textroom_incoming_data(janus_plugin_session *handle, char *label, char *buf, int len);
 void janus_textroom_slow_link(janus_plugin_session *handle, int uplink, int video);
 void janus_textroom_hangup_media(janus_plugin_session *handle);
 void janus_textroom_destroy_session(janus_plugin_session *handle, int *error);
@@ -495,6 +496,7 @@ static janus_plugin janus_textroom_plugin =
 
 		.create_session = janus_textroom_create_session,
 		.handle_message = janus_textroom_handle_message,
+		.handle_admin_message = janus_textroom_handle_admin_message,
 		.setup_media = janus_textroom_setup_media,
 		.incoming_rtp = janus_textroom_incoming_rtp,
 		.incoming_rtcp = janus_textroom_incoming_rtcp,
@@ -1155,6 +1157,55 @@ plugin_response:
 
 }
 
+json_t *janus_textroom_handle_admin_message(json_t *message) {
+	/* Some requests (e.g., 'create' and 'destroy') can be handled via Admin API */
+	int error_code = 0;
+	char error_cause[512];
+	json_t *response = NULL;
+
+	JANUS_VALIDATE_JSON_OBJECT(message, request_parameters,
+		error_code, error_cause, TRUE,
+		JANUS_TEXTROOM_ERROR_MISSING_ELEMENT, JANUS_TEXTROOM_ERROR_INVALID_ELEMENT);
+	if(error_code != 0)
+		goto admin_response;
+	json_t *request = json_object_get(message, "textroom");
+	const char *request_text = json_string_value(request);
+	if(!strcasecmp(request_text, "list")
+			|| !strcasecmp(request_text, "exists")
+			|| !strcasecmp(request_text, "create")
+			|| !strcasecmp(request_text, "edit")
+			|| !strcasecmp(request_text, "destroy")) {
+		janus_plugin_result *result = janus_textroom_handle_incoming_request(NULL, NULL, message, FALSE);
+		if(result == NULL) {
+			JANUS_LOG(LOG_ERR, "JSON error: not an object\n");
+			error_code = JANUS_TEXTROOM_ERROR_INVALID_JSON;
+			g_snprintf(error_cause, 512, "JSON error: not an object");
+			goto admin_response;
+		}
+		response = result->content;
+		result->content = NULL;
+		janus_plugin_result_destroy(result);
+		goto admin_response;
+	} else {
+		JANUS_LOG(LOG_VERB, "Unknown request '%s'\n", request_text);
+		error_code = JANUS_TEXTROOM_ERROR_INVALID_REQUEST;
+		g_snprintf(error_cause, 512, "Unknown request '%s'", request_text);
+	}
+
+admin_response:
+		{
+			if(!response) {
+				/* Prepare JSON error event */
+				response = json_object();
+				json_object_set_new(response, "textroom", json_string("event"));
+				json_object_set_new(response, "error_code", json_integer(error_code));
+				json_object_set_new(response, "error", json_string(error_cause));
+			}
+			return response;
+		}
+
+}
+
 void janus_textroom_setup_media(janus_plugin_session *handle) {
 	JANUS_LOG(LOG_INFO, "WebRTC media is now available\n");
 	if(g_atomic_int_get(&stopping) || !g_atomic_int_get(&initialized))
@@ -1182,7 +1233,7 @@ void janus_textroom_incoming_rtcp(janus_plugin_session *handle, int video, char
 	/* We don't do audio/video */
 }
 
-void janus_textroom_incoming_data(janus_plugin_session *handle, char *buf, int len) {
+void janus_textroom_incoming_data(janus_plugin_session *handle, char *label, char *buf, int len) {
 	if(handle == NULL || handle->stopped || g_atomic_int_get(&stopping) || !g_atomic_int_get(&initialized))
 		return;
 	/* Incoming request from this user: what should we do? */
@@ -1210,7 +1261,9 @@ void janus_textroom_incoming_data(janus_plugin_session *handle, char *buf, int l
 
 /* Helper method to handle incoming messages from the data channel */
 janus_plugin_result *janus_textroom_handle_incoming_request(janus_plugin_session *handle, char *text, json_t *json, gboolean internal) {
-	janus_textroom_session *session = (janus_textroom_session *)handle->plugin_handle;
+	janus_textroom_session *session = NULL;
+	if(handle)
+		session = (janus_textroom_session *)handle->plugin_handle;
 	/* Parse JSON, if needed */
 	json_error_t error;
 	json_t *root = text ? json_loads(text, 0, &error) : json;
@@ -1303,7 +1356,7 @@ janus_plugin_result *janus_textroom_handle_incoming_request(janus_plugin_session
 			janus_textroom_participant *top = g_hash_table_lookup(textroom->participants, to);
 			if(top) {
 				janus_refcount_increase(&top->ref);
-				gateway->relay_data(top->session->handle, msg_text, strlen(msg_text));
+				gateway->relay_data(top->session->handle, NULL, msg_text, strlen(msg_text));
 				janus_refcount_decrease(&top->ref);
 				json_object_set_new(sent, to, json_true());
 			} else {
@@ -1322,7 +1375,7 @@ janus_plugin_result *janus_textroom_handle_incoming_request(janus_plugin_session
 				janus_textroom_participant *top = g_hash_table_lookup(textroom->participants, to);
 				if(top) {
 					janus_refcount_increase(&top->ref);
-					gateway->relay_data(top->session->handle, msg_text, strlen(msg_text));
+					gateway->relay_data(top->session->handle, NULL, msg_text, strlen(msg_text));
 					janus_refcount_decrease(&top->ref);
 					json_object_set_new(sent, to, json_true());
 				} else {
@@ -1342,7 +1395,7 @@ janus_plugin_result *janus_textroom_handle_incoming_request(janus_plugin_session
 					janus_textroom_participant *top = value;
 					JANUS_LOG(LOG_VERB, "  >> To %s in %"SCNu64": %s\n", top->username, room_id, message);
 					janus_refcount_increase(&top->ref);
-					gateway->relay_data(top->session->handle, msg_text, strlen(msg_text));
+					gateway->relay_data(top->session->handle, NULL, msg_text, strlen(msg_text));
 					janus_refcount_decrease(&top->ref);
 				}
 			}
@@ -1468,7 +1521,7 @@ janus_plugin_result *janus_textroom_handle_incoming_request(janus_plugin_session
 				json_object_set_new(event, "display", json_string(display_text));
 			char *event_text = json_dumps(event, json_format);
 			json_decref(event);
-			gateway->relay_data(handle, event_text, strlen(event_text));
+			gateway->relay_data(handle, NULL, event_text, strlen(event_text));
 			/* Broadcast */
 			GHashTableIter iter;
 			gpointer value;
@@ -1479,7 +1532,7 @@ janus_plugin_result *janus_textroom_handle_incoming_request(janus_plugin_session
 					continue;	/* Skip us */
 				janus_refcount_increase(&top->ref);
 				JANUS_LOG(LOG_VERB, "  >> To %s in %"SCNu64"\n", top->username, room_id);
-				gateway->relay_data(top->session->handle, event_text, strlen(event_text));
+				gateway->relay_data(top->session->handle, NULL, event_text, strlen(event_text));
 				/* Take note of this user */
 				json_t *p = json_object();
 				json_object_set_new(p, "username", json_string(top->username));
@@ -1555,7 +1608,7 @@ janus_plugin_result *janus_textroom_handle_incoming_request(janus_plugin_session
 			json_object_set_new(event, "username", json_string(participant->username));
 			char *event_text = json_dumps(event, json_format);
 			json_decref(event);
-			gateway->relay_data(handle, event_text, strlen(event_text));
+			gateway->relay_data(handle, NULL, event_text, strlen(event_text));
 			/* Broadcast */
 			GHashTableIter iter;
 			gpointer value;
@@ -1566,7 +1619,7 @@ janus_plugin_result *janus_textroom_handle_incoming_request(janus_plugin_session
 					continue;	/* Skip us */
 				janus_refcount_increase(&top->ref);
 				JANUS_LOG(LOG_VERB, "  >> To %s in %"SCNu64"\n", top->username, room_id);
-				gateway->relay_data(top->session->handle, event_text, strlen(event_text));
+				gateway->relay_data(top->session->handle, NULL, event_text, strlen(event_text));
 				janus_refcount_decrease(&top->ref);
 			}
 			free(event_text);
@@ -1782,7 +1835,7 @@ janus_plugin_result *janus_textroom_handle_incoming_request(janus_plugin_session
 			while(g_hash_table_iter_next(&iter, NULL, &value)) {
 				janus_textroom_participant *top = value;
 				JANUS_LOG(LOG_VERB, "  >> To %s in %"SCNu64"\n", top->username, room_id);
-				gateway->relay_data(top->session->handle, event_text, strlen(event_text));
+				gateway->relay_data(top->session->handle, NULL, event_text, strlen(event_text));
 			}
 			free(event_text);
 		}
@@ -1984,7 +2037,7 @@ janus_plugin_result *janus_textroom_handle_incoming_request(janus_plugin_session
 			json_t *info = json_object();
 			json_object_set_new(info, "event", json_string("created"));
 			json_object_set_new(info, "room", json_integer(room_id));
-			gateway->notify_event(&janus_textroom_plugin, session->handle, info);
+			gateway->notify_event(&janus_textroom_plugin, session ? session->handle : NULL, info);
 		}
 	} else if(!strcasecmp(request_text, "exists")) {
 		JANUS_VALIDATE_JSON_OBJECT(root, room_parameters,
@@ -2112,7 +2165,7 @@ janus_plugin_result *janus_textroom_handle_incoming_request(janus_plugin_session
 			json_t *info = json_object();
 			json_object_set_new(info, "event", json_string("edited"));
 			json_object_set_new(info, "room", json_integer(room_id));
-			gateway->notify_event(&janus_textroom_plugin, session->handle, info);
+			gateway->notify_event(&janus_textroom_plugin, session ? session->handle : NULL, info);
 		}
 	} else if(!strcasecmp(request_text, "destroy")) {
 		JANUS_VALIDATE_JSON_OBJECT(root, destroy_parameters,
@@ -2175,7 +2228,7 @@ janus_plugin_result *janus_textroom_handle_incoming_request(janus_plugin_session
 			json_object_set_new(event, "room", json_integer(textroom->room_id));
 			char *event_text = json_dumps(event, json_format);
 			json_decref(event);
-			gateway->relay_data(handle, event_text, strlen(event_text));
+			gateway->relay_data(handle, NULL, event_text, strlen(event_text));
 			/* Broadcast */
 			GHashTableIter iter;
 			gpointer value;
@@ -2184,7 +2237,7 @@ janus_plugin_result *janus_textroom_handle_incoming_request(janus_plugin_session
 				janus_textroom_participant *top = value;
 				janus_refcount_increase(&top->ref);
 				JANUS_LOG(LOG_VERB, "  >> To %s in %"SCNu64"\n", top->username, room_id);
-				gateway->relay_data(top->session->handle, event_text, strlen(event_text));
+				gateway->relay_data(top->session->handle, NULL, event_text, strlen(event_text));
 				janus_mutex_lock(&top->session->mutex);
 				g_hash_table_remove(top->session->rooms, &room_id);
 				janus_mutex_unlock(&top->session->mutex);
@@ -2209,7 +2262,7 @@ janus_plugin_result *janus_textroom_handle_incoming_request(janus_plugin_session
 			json_t *info = json_object();
 			json_object_set_new(info, "event", json_string("destroyed"));
 			json_object_set_new(info, "room", json_integer(room_id));
-			gateway->notify_event(&janus_textroom_plugin, session->handle, info);
+			gateway->notify_event(&janus_textroom_plugin, session ? session->handle : NULL, info);
 		}
 	} else {
 		JANUS_LOG(LOG_ERR, "Unsupported request %s\n", request_text);
@@ -2239,7 +2292,7 @@ msg_response:
 					/* Reply via data channels */
 					char *reply_text = json_dumps(reply, json_format);
 					json_decref(reply);
-					gateway->relay_data(handle, reply_text, strlen(reply_text));
+					gateway->relay_data(handle, NULL, reply_text, strlen(reply_text));
 					free(reply_text);
 				} else {
 					/* Reply via Janus API */
