@@ -290,7 +290,7 @@ struct janus_plugin_result *janus_videocall_handle_message(janus_plugin_session
 void janus_videocall_setup_media(janus_plugin_session *handle);
 void janus_videocall_incoming_rtp(janus_plugin_session *handle, int video, char *buf, int len);
 void janus_videocall_incoming_rtcp(janus_plugin_session *handle, int video, char *buf, int len);
-void janus_videocall_incoming_data(janus_plugin_session *handle, char *buf, int len);
+void janus_videocall_incoming_data(janus_plugin_session *handle, char *label, char *buf, int len);
 void janus_videocall_slow_link(janus_plugin_session *handle, int uplink, int video);
 void janus_videocall_hangup_media(janus_plugin_session *handle);
 void janus_videocall_destroy_session(janus_plugin_session *handle, int *error);
@@ -375,8 +375,8 @@ typedef struct janus_videocall_session {
 	struct janus_videocall_session *peer;
 	janus_rtp_switching_context context;
 	uint32_t ssrc[3];		/* Only needed in case VP8 (or H.264) simulcasting is involved */
+	char *rid[3];			/* Only needed if simulcasting is rid-based */
 	janus_rtp_simulcasting_context sim_context;
-	int rtpmapid_extmap_id;	/* Only needed for debugging in case Firefox's RID-based simulcasting is involved */
 	janus_vp8_simulcast_context vp8_context;
 	janus_recorder *arc;	/* The Janus recorder instance for this user's audio, if enabled */
 	janus_recorder *vrc;	/* The Janus recorder instance for this user's video, if enabled */
@@ -629,10 +629,10 @@ json_t *janus_videocall_query_session(janus_plugin_session *handle) {
 		json_object_set_new(info, "bitrate", json_integer(session->bitrate));
 		json_object_set_new(info, "slowlink_count", json_integer(session->slowlink_count));
 	}
-	if(session->ssrc[0] != 0) {
+	if(session->ssrc[0] != 0 || session->rid[0] != NULL) {
 		json_object_set_new(info, "simulcast", json_true());
 	}
-	if(peer && peer->ssrc[0] != 0) {
+	if(peer && (peer->ssrc[0] != 0 || peer->rid[0] != NULL)) {
 		json_object_set_new(info, "simulcast-peer", json_true());
 		json_object_set_new(info, "substream", json_integer(session->sim_context.substream));
 		json_object_set_new(info, "substream-target", json_integer(session->sim_context.substream_target));
@@ -709,19 +709,7 @@ void janus_videocall_incoming_rtp(janus_plugin_session *handle, int video, char
 		}
 		if(g_atomic_int_get(&session->destroyed) || g_atomic_int_get(&peer->destroyed))
 			return;
-		if(video && session->video_active && session->rtpmapid_extmap_id != -1) {
-			/* FIXME Just a way to debug Firefox simulcasting */
-			janus_rtp_header *header = (janus_rtp_header *)buf;
-			uint32_t seq_number = ntohs(header->seq_number);
-			uint32_t timestamp = ntohl(header->timestamp);
-			uint32_t ssrc = ntohl(header->ssrc);
-			char sdes_item[16];
-			if(janus_rtp_header_extension_parse_rtp_stream_id(buf, len, session->rtpmapid_extmap_id, sdes_item, sizeof(sdes_item)) == 0) {
-				JANUS_LOG(LOG_DBG, "%"SCNu32"/%"SCNu16"/%"SCNu32"/%d: RTP stream ID extension: %s\n",
-					ssrc, seq_number, timestamp, header->padding, sdes_item);
-			}
-		}
-		if(video && session->video_active && session->ssrc[0] != 0) {
+		if(video && session->video_active && (session->ssrc[0] != 0 || session->rid[0] != NULL)) {
 			/* Handle simulcast: backup the header information first */
 			janus_rtp_header *header = (janus_rtp_header *)buf;
 			uint32_t seq_number = ntohs(header->seq_number);
@@ -730,7 +718,7 @@ void janus_videocall_incoming_rtp(janus_plugin_session *handle, int video, char
 			/* Process this packet: don't relay if it's not the SSRC/layer we wanted to handle
 			 * The caveat is that the targets in OUR simulcast context are the PEER's targets */
 			gboolean relay = janus_rtp_simulcasting_context_process_rtp(&peer->sim_context,
-				buf, len, session->ssrc, session->vcodec, &peer->context);
+				buf, len, session->ssrc, session->rid, session->vcodec, &peer->context);
 			/* Do we need to drop this? */
 			if(!relay)
 				return;
@@ -813,7 +801,9 @@ void janus_videocall_incoming_rtcp(janus_plugin_session *handle, int video, char
 		guint32 bitrate = janus_rtcp_get_remb(buf, len);
 		if(bitrate > 0) {
 			/* If a REMB arrived, make sure we cap it to our configuration, and send it as a video RTCP */
-			if(session->bitrate > 0)
+			if(session->bitrate == 0)	/* No limit ~= 10000000 */
+				janus_rtcp_cap_remb(buf, len, 10000000);
+			else
 				janus_rtcp_cap_remb(buf, len, session->bitrate);
 			gateway->relay_rtcp(peer->handle, 1, buf, len);
 			return;
@@ -822,7 +812,7 @@ void janus_videocall_incoming_rtcp(janus_plugin_session *handle, int video, char
 	}
 }
 
-void janus_videocall_incoming_data(janus_plugin_session *handle, char *buf, int len) {
+void janus_videocall_incoming_data(janus_plugin_session *handle, char *label, char *buf, int len) {
 	if(handle == NULL || g_atomic_int_get(&handle->stopped) || g_atomic_int_get(&stopping) || !g_atomic_int_get(&initialized))
 		return;
 	if(gateway) {
@@ -847,7 +837,7 @@ void janus_videocall_incoming_data(janus_plugin_session *handle, char *buf, int
 		/* Save the frame if we're recording */
 		janus_recorder_save_frame(session->drc, buf, len);
 		/* Forward the packet to the peer */
-		gateway->relay_data(peer->handle, text, strlen(text));
+		gateway->relay_data(peer->handle, label, text, strlen(text));
 		g_free(text);
 	}
 }
@@ -866,12 +856,12 @@ void janus_videocall_slow_link(janus_plugin_session *handle, int uplink, int vid
 	session->slowlink_count++;
 	if(uplink && !video && !session->audio_active) {
 		/* We're not relaying audio and the peer is expecting it, so NACKs are normal */
-		JANUS_LOG(LOG_VERB, "Getting a lot of NACKs (slow uplink) for audio, but that's expected, a configure disabled the audio forwarding\n");
+		JANUS_LOG(LOG_VERB, "Getting a lot of lost packets (slow uplink) for audio, but that's expected, a configure disabled the audio forwarding\n");
 	} else if(uplink && video && !session->video_active) {
 		/* We're not relaying video and the peer is expecting it, so NACKs are normal */
-		JANUS_LOG(LOG_VERB, "Getting a lot of NACKs (slow uplink) for video, but that's expected, a configure disabled the video forwarding\n");
+		JANUS_LOG(LOG_VERB, "Getting a lot of lost packets (slow uplink) for video, but that's expected, a configure disabled the video forwarding\n");
 	} else {
-		JANUS_LOG(LOG_WARN, "Getting a lot of NACKs (slow %s) for %s\n",
+		JANUS_LOG(LOG_WARN, "Getting a lot of lost packets (slow %s) for %s\n",
 			uplink ? "uplink" : "downlink", video ? "video" : "audio");
 		if(!uplink) {
 			/* Send an event on the handle to notify the application: it's
@@ -881,7 +871,9 @@ void janus_videocall_slow_link(janus_plugin_session *handle, int uplink, int vid
 			/* Also add info on what the current bitrate cap is */
 			json_t *result = json_object();
 			json_object_set_new(result, "event", json_string("slow_link"));
-			json_object_set_new(result, "current-bitrate", json_integer(session->bitrate));
+			json_object_set_new(result, "media", json_string(video ? "video" : "audio"));
+			if(video)
+				json_object_set_new(result, "current-bitrate", json_integer(session->bitrate));
 			json_object_set_new(event, "result", result);
 			gateway->push_event(session->handle, &janus_videocall_plugin, NULL, event, NULL);
 			json_decref(event);
@@ -962,6 +954,12 @@ void janus_videocall_hangup_media(janus_plugin_session *handle) {
 	session->acodec = JANUS_AUDIOCODEC_NONE;
 	session->vcodec = JANUS_VIDEOCODEC_NONE;
 	session->bitrate = 0;
+	int i=0;
+	for(i=0; i<3; i++) {
+		session->ssrc[i] = 0;
+		g_free(session->rid[i]);
+		session->rid[i] = NULL;
+	}
 	janus_rtp_switching_context_reset(&session->context);
 	janus_rtp_simulcasting_context_reset(&session->sim_context);
 	janus_vp8_simulcast_context_reset(&session->vp8_context);
@@ -1204,11 +1202,12 @@ static void *janus_videocall_handler(void *data) {
 				JANUS_LOG(LOG_VERB, "This is involving a negotiation (%s) as well:\n%s\n", msg_sdp_type, msg_sdp);
 				/* Check if this user will simulcast */
 				json_t *msg_simulcast = json_object_get(msg->jsep, "simulcast");
-				if(msg_simulcast && janus_get_codec_pt(msg_sdp, "vp8") > 0) {
+				if(msg_simulcast) {
 					JANUS_LOG(LOG_VERB, "VideoCall caller (%s) is going to do simulcasting\n", session->username);
-					session->ssrc[0] = json_integer_value(json_object_get(msg_simulcast, "ssrc-0"));
-					session->ssrc[1] = json_integer_value(json_object_get(msg_simulcast, "ssrc-1"));
-					session->ssrc[2] = json_integer_value(json_object_get(msg_simulcast, "ssrc-2"));
+					int rid_ext_id = -1, framemarking_ext_id = -1;
+					janus_rtp_simulcasting_prepare(msg_simulcast, &rid_ext_id, &framemarking_ext_id, session->ssrc, session->rid);
+					session->sim_context.rid_ext_id = rid_ext_id;
+					session->sim_context.framemarking_ext_id = framemarking_ext_id;
 				}
 				/* Send SDP to our peer */
 				json_t *call = json_object();
@@ -1273,13 +1272,16 @@ static void *janus_videocall_handler(void *data) {
 				session->ssrc[1] = json_integer_value(json_object_get(msg_simulcast, "ssrc-1"));
 				session->ssrc[2] = json_integer_value(json_object_get(msg_simulcast, "ssrc-2"));
 			} else {
-				session->ssrc[0] = 0;
-				session->ssrc[1] = 0;
-				session->ssrc[2] = 0;
-				if(peer) {
-					peer->ssrc[0] = 0;
-					peer->ssrc[1] = 0;
-					peer->ssrc[2] = 0;
+				int i=0;
+				for(i=0; i<3; i++) {
+					session->ssrc[i] = 0;
+					g_free(session->rid[0]);
+					session->rid[0] = NULL;
+					if(peer) {
+						peer->ssrc[i] = 0;
+						g_free(peer->rid[0]);
+						peer->rid[0] = NULL;
+					}
 				}
 			}
 			/* Check which codecs we ended up using */
@@ -1325,11 +1327,11 @@ static void *janus_videocall_handler(void *data) {
 				gateway->notify_event(&janus_videocall_plugin, session->handle, info);
 			}
 			/* Is simulcasting involved on either side? */
-			if(session->ssrc[0] && session->ssrc[1]) {
+			if(session->ssrc[0] || session->rid[0]) {
 				peer->sim_context.substream_target = 2;	/* Let's aim for the highest quality */
 				peer->sim_context.templayer_target = 2;	/* Let's aim for all temporal layers */
 			}
-			if(peer->ssrc[0] && peer->ssrc[1]) {
+			if(peer->ssrc[0] || peer->rid[0]) {
 				session->sim_context.substream_target = 2;	/* Let's aim for the highest quality */
 				session->sim_context.templayer_target = 2;	/* Let's aim for all temporal layers */
 			}
