@@ -142,6 +142,13 @@ function handleMessage(id, tr, msg, jsep)
 	end
 end
 
+function handleAdminMessage(message)
+	-- This is just to showcase how you can handle incoming messages
+	-- coming from the Admin API: we return the same message as a test
+	logger.print("Got admin message: " .. dumpTable(message))
+	return message;
+end
+
 function setupMedia(id)
 	-- WebRTC is now available
 	logger.print("WebRTC PeerConnection is up for session: " .. id)
@@ -226,7 +233,7 @@ function processRequest(id, msg)
 			"audio", "opus", "/tmp", fnbase .. "-audio",
 			"video", "vp8", "/tmp", fnbase .. "-video",
 			"data", "text", "/tmp", fnbase .. "-data"
-		) 
+		)
 	elseif msg["record"] == false then
 		stopRecording(id, "audio", "video", "data")
 	end
