@@ -4,7 +4,7 @@
  * \brief    Post-processing to generate .opus files
  * \details  Implementation of the post-processing code (based on libogg)
  * needed to generate .opus files out of Opus RTP frames.
- * 
+ *
  * \ingroup postprocessing
  * \ref postprocessing
  */
@@ -24,6 +24,7 @@
 #include "pp-opus.h"
 #include "pp-opus-silence.h"
 #include "../debug.h"
+#include "../version.h"
 
 
 /* OGG/Opus helpers */
@@ -33,14 +34,14 @@ ogg_stream_state *stream = NULL;
 void le32(unsigned char *p, int v);
 void le16(unsigned char *p, int v);
 ogg_packet *op_opushead(void);
-ogg_packet *op_opustags(void);
+ogg_packet *op_opustags(char *metadata);
 ogg_packet *op_from_pkt(const unsigned char *pkt, int len);
 void op_free(ogg_packet *op);
 int ogg_write(void);
 int ogg_flush(void);
 
 
-int janus_pp_opus_create(char *destination) {
+int janus_pp_opus_create(char *destination, char *metadata) {
 	stream = g_malloc0(sizeof(ogg_stream_state));
 	if(ogg_stream_init(stream, rand()) < 0) {
 		JANUS_LOG(LOG_ERR, "Couldn't initialize Ogg stream state\n");
@@ -56,7 +57,7 @@ int janus_pp_opus_create(char *destination) {
 	ogg_packet *op = op_opushead();
 	ogg_stream_packetin(stream, op);
 	op_free(op);
-	op = op_opustags();
+	op = op_opustags(metadata);
 	ogg_stream_packetin(stream, op);
 	op_free(op);
 	ogg_flush();
@@ -97,15 +98,25 @@ int janus_pp_opus_process(FILE *file, janus_pp_frame_packet *list, int *working)
 			tmp = tmp->next;
 			continue;
 		}
+		if(tmp->audiolevel != -1) {
+			JANUS_LOG(LOG_VERB, "Audio level: %d dB\n", tmp->audiolevel);
+		}
 		guint16 diff = tmp->prev == NULL ? 1 : (tmp->seq - tmp->prev->seq);
 		len = 0;
 		/* RTP payload */
 		offset = tmp->offset+12+tmp->skip;
 		fseek(file, offset, SEEK_SET);
 		len = tmp->len-12-tmp->skip;
+		if(len < 1) {
+			tmp = tmp->next;
+			continue;
+		}
 		bytes = fread(buffer, sizeof(char), len, file);
-		if(bytes != len)
+		if(bytes != len) {
 			JANUS_LOG(LOG_WARN, "Didn't manage to read all the bytes we needed (%d < %d)...\n", bytes, len);
+			tmp = tmp->next;
+			continue;
+		}
 		if(last_seq == 0)
 			last_seq = tmp->seq;
 		if(tmp->seq < last_seq) {
@@ -113,7 +124,7 @@ int janus_pp_opus_process(FILE *file, janus_pp_frame_packet *list, int *working)
 			steps++;
 		}
 		ogg_packet *op = op_from_pkt((const unsigned char *)buffer, bytes);
-		pos = (tmp->ts - list->ts) / 48 / 20;
+		pos = (tmp->ts - list->ts) / 48 / 20 + 1;
 		JANUS_LOG(LOG_VERB, "pos: %06"SCNu64", writing %d bytes out of %d (seq=%"SCNu16", step=%"SCNu16", ts=%"SCNu64", time=%"SCNu64"s)\n",
 			pos, bytes, tmp->len, tmp->seq, diff, tmp->ts, (tmp->ts-list->ts)/48000);
 		op->granulepos = 960*(pos); /* FIXME: get this from the toc byte */
@@ -179,17 +190,30 @@ ogg_packet *op_opushead(void) {
 }
 
 /* Manufacture a generic OpusTags packet */
-ogg_packet *op_opustags(void) {
+ogg_packet *op_opustags(char *metadata) {
 	const char *identifier = "OpusTags";
-	const char *vendor = "Janus post-processing";
+	const char *desc = "DESCRIPTION=";
+	char vendor[256];
+	g_snprintf(vendor, sizeof(vendor), "Janus post-processor %s", janus_version_string);
 	int size = strlen(identifier) + 4 + strlen(vendor) + 4;
+	int dlen = strlen(desc), mlen = metadata ? strlen(metadata) : 0;
+	if(mlen > 0)
+		size += (4+dlen+mlen);
 	unsigned char *data = g_malloc(size);
 	ogg_packet *op = g_malloc(sizeof(*op));
 
+	/* Write down the tags */
 	memcpy(data, identifier, 8);
 	le32(data + 8, strlen(vendor));
 	memcpy(data + 12, vendor, strlen(vendor));
-	le32(data + 12 + strlen(vendor), 0);
+	le32(data + 12 + strlen(vendor), mlen > 0 ? 1 : 0);
+	/* Check if we have metadata to write down: we'll use the "DESCRIPTION" tag */
+	if(metadata && strlen(metadata) > 0) {
+		/* Add a single comment */
+		le32(data + 12 + strlen(vendor) + 4, dlen+mlen);
+		memcpy(data + 12 + strlen(vendor) + 8, desc, dlen);
+		memcpy(data + 12 + strlen(vendor) + 8 + dlen, metadata, mlen);
+	}
 
 	op->packet = data;
 	op->bytes = size;
