@@ -346,8 +346,10 @@ int janus_sdp_process(void *ice_handle, janus_sdp *remote_sdp, gboolean update)
 			tempA = tempA->next;
 		}
 		if(mlines == 1) {
-			if(!ruser || !rpass || !rfingerprint || !rhashing) {
+			if(!ruser || !rpass || (janus_is_webrtc_encryption_enabled() && (!rfingerprint || !rhashing))) {
 				/* Missing mandatory information, failure... */
+				JANUS_LOG(LOG_ERR, "[%"SCNu64"] SDP missing mandatory information\n", handle->handle_id);
+				JANUS_LOG(LOG_ERR, "[%"SCNu64"] %p, %p, %p, %p\n", handle->handle_id, ruser, rpass, rfingerprint, rhashing);
 				if(ruser)
 					g_free(ruser);
 				ruser = NULL;
@@ -362,6 +364,13 @@ int janus_sdp_process(void *ice_handle, janus_sdp *remote_sdp, gboolean update)
 				rfingerprint = NULL;
 				return -2;
 			}
+			/* If we received the ICE credentials for the first time, enforce them */
+			if(ruser && !stream->ruser && rpass && !stream->rpass) {
+				JANUS_LOG(LOG_VERB, "[%"SCNu64"] Setting remote credentials...\n", handle->handle_id);
+				if(!nice_agent_set_remote_credentials(handle->agent, handle->stream_id, ruser, rpass)) {
+					JANUS_LOG(LOG_ERR, "[%"SCNu64"] Failed to set remote credentials!\n", handle->handle_id);
+				}
+			} else
 			/* If this is a renegotiation, check if this is an ICE restart */
 			if((ruser && stream->ruser && strcmp(ruser, stream->ruser)) ||
 					(rpass && stream->rpass && strcmp(rpass, stream->rpass))) {
@@ -370,10 +379,12 @@ int janus_sdp_process(void *ice_handle, janus_sdp *remote_sdp, gboolean update)
 				janus_flags_set(&handle->webrtc_flags, JANUS_ICE_HANDLE_WEBRTC_ICE_RESTART);
 			}
 			/* Store fingerprint and hashing */
-			g_free(stream->remote_hashing);
-			stream->remote_hashing = g_strdup(rhashing);
-			g_free(stream->remote_fingerprint);
-			stream->remote_fingerprint = g_strdup(rfingerprint);
+			if(janus_is_webrtc_encryption_enabled()) {
+				g_free(stream->remote_hashing);
+				stream->remote_hashing = g_strdup(rhashing);
+				g_free(stream->remote_fingerprint);
+				stream->remote_fingerprint = g_strdup(rfingerprint);
+			}
 			/* Store the ICE username and password for this stream */
 			g_free(stream->ruser);
 			stream->ruser = g_strdup(ruser);
@@ -384,8 +395,8 @@ int janus_sdp_process(void *ice_handle, janus_sdp *remote_sdp, gboolean update)
 		tempA = m->attributes;
 		while(tempA) {
 			janus_sdp_attribute *a = (janus_sdp_attribute *)tempA->data;
-			if(a->name && !strcasecmp(a->name, "rid")) {
-				/* This attribute is used by Firefox for simulcasting */
+			if(a->name && !strcasecmp(a->name, "rid") && a->value) {
+				/* This attribute is used for simulcasting */
 				char rid[16];
 				if(sscanf(a->value, "%15s send", rid) != 1) {
 					JANUS_LOG(LOG_ERR, "[%"SCNu64"] Failed to parse rid attribute...\n", handle->handle_id);
@@ -401,6 +412,9 @@ int janus_sdp_process(void *ice_handle, janus_sdp *remote_sdp, gboolean update)
 						JANUS_LOG(LOG_WARN, "[%"SCNu64"] Too many RTP Stream IDs, ignoring '%s'...\n", handle->handle_id, rid);
 					}
 				}
+			} else if(a->name && !strcasecmp(a->name, "simulcast") && a->value) {
+				/* Firefox and Chrome signal simulcast support differently */
+				stream->legacy_rid = strstr(a->value, "rid=") ? TRUE : FALSE;
 			}
 			tempA = tempA->next;
 		}
@@ -598,7 +612,7 @@ int janus_sdp_parse_candidate(void *ice_stream, const char *candidate, int trick
 	if(handle == NULL)
 		return -2;
 	janus_ice_component *component = NULL;
-	if(strstr(candidate, "end-of-candidates")) {
+	if(strlen(candidate) == 0 || strstr(candidate, "end-of-candidates")) {
 		/* FIXME Should we do something with this? */
 		JANUS_LOG(LOG_VERB, "[%"SCNu64"] end-of-candidates received\n", handle->handle_id);
 		return 0;
@@ -738,7 +752,7 @@ int janus_sdp_parse_candidate(void *ice_stream, const char *candidate, int trick
 					c->transport = ctype;
 				}
 #endif
-				strncpy(c->foundation, rfoundation, NICE_CANDIDATE_MAX_FOUNDATION);
+				g_strlcpy(c->foundation, rfoundation, NICE_CANDIDATE_MAX_FOUNDATION);
 				c->priority = rpriority;
 				nice_address_set_from_string(&c->addr, rip);
 				nice_address_set_port(&c->addr, rport);
@@ -827,6 +841,10 @@ int janus_sdp_parse_ssrc_group(void *ice_stream, const char *group_attr, int vid
 		return -2;
 	if(!video)
 		return -3;
+	if(stream->rid[0] != NULL) {
+		/* Simulcasting is rid-based, don't parse SSRCs for now */
+		return 0;
+	}
 	gboolean fid = strstr(group_attr, "FID") != NULL;
 	gboolean sim = strstr(group_attr, "SIM") != NULL;
 	guint64 ssrc = 0;
@@ -850,7 +868,7 @@ int janus_sdp_parse_ssrc_group(void *ice_stream, const char *group_attr, int vid
 								stream->video_ssrc_peer_new[0] = ssrc;
 								JANUS_LOG(LOG_VERB, "[%"SCNu64"] Peer video SSRC: %"SCNu32"\n", handle->handle_id, stream->video_ssrc_peer_new[0]);
 							} else {
-								/* We already have a video SSRC: check if RID is involved, and we'll keep track of this for simulcasting */
+								/* We already have a video SSRC: check if rid is involved, and we'll keep track of this for simulcasting */
 								if(stream->rid[0]) {
 									if(stream->video_ssrc_peer_new[1] == 0) {
 										stream->video_ssrc_peer_new[1] = ssrc;
@@ -920,27 +938,17 @@ int janus_sdp_parse_ssrc(void *ice_stream, const char *ssrc_attr, int video) {
 	if(ssrc == 0 || ssrc > G_MAXUINT32)
 		return -3;
 	if(video) {
-		if(stream->video_ssrc_peer_new[0] == ssrc || stream->video_ssrc_peer_new[1] == ssrc
-				|| stream->video_ssrc_peer_new[2] == ssrc) {
-			JANUS_LOG(LOG_HUGE, "[%"SCNu64"] Already parsed this SSRC: %"SCNu64"\n", handle->handle_id, ssrc);
+		if(stream->rid[0] != NULL) {
+			/* Simulcasting is rid-based, only keep track of a single SSRC for fallback */
+			if(stream->video_ssrc_peer_temp == 0) {
+				stream->video_ssrc_peer_temp = ssrc;
+				JANUS_LOG(LOG_WARN, "[%"SCNu64"] Peer video fallback SSRC: %"SCNu32"\n", handle->handle_id, stream->video_ssrc_peer_temp);
+			}
 			return 0;
 		}
 		if(stream->video_ssrc_peer_new[0] == 0) {
 			stream->video_ssrc_peer_new[0] = ssrc;
 			JANUS_LOG(LOG_VERB, "[%"SCNu64"] Peer video SSRC: %"SCNu32"\n", handle->handle_id, stream->video_ssrc_peer_new[0]);
-		} else {
-			/* We already have a video SSRC: check if RID is involved, and we'll keep track of this for simulcasting */
-			if(stream->rid[0]) {
-				if(stream->video_ssrc_peer_new[1] == 0) {
-					stream->video_ssrc_peer_new[1] = ssrc;
-					JANUS_LOG(LOG_VERB, "[%"SCNu64"] Peer video SSRC (sim-1): %"SCNu32"\n", handle->handle_id, stream->video_ssrc_peer_new[1]);
-				} else if(stream->video_ssrc_peer_new[2] == 0) {
-					stream->video_ssrc_peer_new[2] = ssrc;
-					JANUS_LOG(LOG_VERB, "[%"SCNu64"] Peer video SSRC (sim-2): %"SCNu32"\n", handle->handle_id, stream->video_ssrc_peer_new[2]);
-				} else {
-					JANUS_LOG(LOG_WARN, "[%"SCNu64"] Don't know what to do with video SSRC: %"SCNu64"\n", handle->handle_id, ssrc);
-				}
-			}
 		}
 	} else {
 		if(stream->audio_ssrc_peer_new == 0) {
@@ -1028,16 +1036,29 @@ int janus_sdp_anonymize(janus_sdp *anon) {
 					|| !strcasecmp(a->name, "msid")
 					|| !strcasecmp(a->name, "msid-semantic")
 					|| !strcasecmp(a->name, "rid")
+					|| !strcasecmp(a->name, "simulcast")
 					|| !strcasecmp(a->name, "rtcp")
 					|| !strcasecmp(a->name, "rtcp-mux")
 					|| !strcasecmp(a->name, "rtcp-rsize")
 					|| !strcasecmp(a->name, "candidate")
+					|| !strcasecmp(a->name, "end-of-candidates")
 					|| !strcasecmp(a->name, "ssrc")
 					|| !strcasecmp(a->name, "ssrc-group")
 					|| !strcasecmp(a->name, "sctpmap")
 					|| !strcasecmp(a->name, "sctp-port")
-					|| !strcasecmp(a->name, "max-message-size")
-					|| (a->value && strstr(a->value, "urn:ietf:params:rtp-hdrext:sdes:"))) {
+					|| !strcasecmp(a->name, "max-message-size")) {
+				m->attributes = g_list_remove(m->attributes, a);
+				tempA = m->attributes;
+				janus_sdp_attribute_destroy(a);
+				continue;
+			}
+			tempA = tempA->next;
+		}
+		/* We don't support encrypted RTP extensions yet, so get rid of them */
+		tempA = m->attributes;
+		while(tempA) {
+			janus_sdp_attribute *a = (janus_sdp_attribute *)tempA->data;
+			if(a->value && strstr(a->value, JANUS_RTP_EXTMAP_ENCRYPTED)) {
 				m->attributes = g_list_remove(m->attributes, a);
 				tempA = m->attributes;
 				janus_sdp_attribute_destroy(a);
@@ -1085,6 +1106,8 @@ char *janus_sdp_merge(void *ice_handle, janus_sdp *anon, gboolean offer) {
 	if(stream == NULL)
 		return NULL;
 	char *rtp_profile = handle->rtp_profile ? handle->rtp_profile : (char *)"UDP/TLS/RTP/SAVPF";
+	if(!janus_is_webrtc_encryption_enabled())
+		rtp_profile = (char *)"RTP/AVPF";
 	gboolean ipv4 = !strstr(janus_get_public_ip(), ":");
 	/* Origin o= */
 	gint64 sessid = janus_get_real_time();
@@ -1329,19 +1352,23 @@ char *janus_sdp_merge(void *ice_handle, janus_sdp *anon, gboolean offer) {
 		g_free(password);
 		a = janus_sdp_attribute_create("ice-options", "trickle");
 		m->attributes = g_list_insert_before(m->attributes, first, a);
-		a = janus_sdp_attribute_create("fingerprint", "sha-256 %s", janus_dtls_get_local_fingerprint());
-		m->attributes = g_list_insert_before(m->attributes, first, a);
-		a = janus_sdp_attribute_create("setup", "%s", janus_get_dtls_srtp_role(offer ? JANUS_DTLS_ROLE_ACTPASS : stream->dtls_role));
-		m->attributes = g_list_insert_before(m->attributes, first, a);
+		if(janus_is_webrtc_encryption_enabled()) {
+			a = janus_sdp_attribute_create("fingerprint", "sha-256 %s", janus_dtls_get_local_fingerprint());
+			m->attributes = g_list_insert_before(m->attributes, first, a);
+			a = janus_sdp_attribute_create("setup", "%s", janus_get_dtls_srtp_role(offer ? JANUS_DTLS_ROLE_ACTPASS : stream->dtls_role));
+			m->attributes = g_list_insert_before(m->attributes, first, a);
+		}
 		/* Add last attributes, rtcp and ssrc (msid) */
-		if(m->type == JANUS_SDP_VIDEO && janus_flags_is_set(&handle->webrtc_flags, JANUS_ICE_HANDLE_WEBRTC_RFC4588_RTX)) {
+		if(m->type == JANUS_SDP_VIDEO && janus_flags_is_set(&handle->webrtc_flags, JANUS_ICE_HANDLE_WEBRTC_RFC4588_RTX) &&
+				(m->direction == JANUS_SDP_DEFAULT || m->direction == JANUS_SDP_SENDRECV || m->direction == JANUS_SDP_SENDONLY)) {
 			/* Add FID group to negotiate the RFC4588 stuff */
 			a = janus_sdp_attribute_create("ssrc-group", "FID %"SCNu32" %"SCNu32, stream->video_ssrc, stream->video_ssrc_rtx);
 			m->attributes = g_list_append(m->attributes, a);
 		}
-		if(m->type == JANUS_SDP_AUDIO &&
-				(m->direction == JANUS_SDP_DEFAULT || m->direction == JANUS_SDP_SENDRECV || m->direction == JANUS_SDP_SENDONLY)) {
-			a = janus_sdp_attribute_create("ssrc", "%"SCNu32" cname:janusaudio", stream->audio_ssrc);
+		if(m->type == JANUS_SDP_AUDIO) {
+			a = janus_sdp_attribute_create("msid", "janus janusa0");
+			m->attributes = g_list_append(m->attributes, a);
+			a = janus_sdp_attribute_create("ssrc", "%"SCNu32" cname:janus", stream->audio_ssrc);
 			m->attributes = g_list_append(m->attributes, a);
 			a = janus_sdp_attribute_create("ssrc", "%"SCNu32" msid:janus janusa0", stream->audio_ssrc);
 			m->attributes = g_list_append(m->attributes, a);
@@ -1349,9 +1376,10 @@ char *janus_sdp_merge(void *ice_handle, janus_sdp *anon, gboolean offer) {
 			m->attributes = g_list_append(m->attributes, a);
 			a = janus_sdp_attribute_create("ssrc", "%"SCNu32" label:janusa0", stream->audio_ssrc);
 			m->attributes = g_list_append(m->attributes, a);
-		} else if(m->type == JANUS_SDP_VIDEO &&
-				(m->direction == JANUS_SDP_DEFAULT || m->direction == JANUS_SDP_SENDRECV || m->direction == JANUS_SDP_SENDONLY)) {
-			a = janus_sdp_attribute_create("ssrc", "%"SCNu32" cname:janusvideo", stream->video_ssrc);
+		} else if(m->type == JANUS_SDP_VIDEO) {
+			a = janus_sdp_attribute_create("msid", "janus janusv0");
+			m->attributes = g_list_append(m->attributes, a);
+			a = janus_sdp_attribute_create("ssrc", "%"SCNu32" cname:janus", stream->video_ssrc);
 			m->attributes = g_list_append(m->attributes, a);
 			a = janus_sdp_attribute_create("ssrc", "%"SCNu32" msid:janus janusv0", stream->video_ssrc);
 			m->attributes = g_list_append(m->attributes, a);
@@ -1361,7 +1389,7 @@ char *janus_sdp_merge(void *ice_handle, janus_sdp *anon, gboolean offer) {
 			m->attributes = g_list_append(m->attributes, a);
 			if(janus_flags_is_set(&handle->webrtc_flags, JANUS_ICE_HANDLE_WEBRTC_RFC4588_RTX)) {
 				/* Add rtx SSRC group to negotiate the RFC4588 stuff */
-				a = janus_sdp_attribute_create("ssrc", "%"SCNu32" cname:janusvideo", stream->video_ssrc_rtx);
+				a = janus_sdp_attribute_create("ssrc", "%"SCNu32" cname:janus", stream->video_ssrc_rtx);
 				m->attributes = g_list_append(m->attributes, a);
 				a = janus_sdp_attribute_create("ssrc", "%"SCNu32" msid:janus janusv0", stream->video_ssrc_rtx);
 				m->attributes = g_list_append(m->attributes, a);
@@ -1388,7 +1416,11 @@ char *janus_sdp_merge(void *ice_handle, janus_sdp *anon, gboolean offer) {
 					g_strlcat(rids, stream->rid[i], sizeof(rids));
 				}
 			}
-			a = janus_sdp_attribute_create("simulcast", " recv rid=%s", rids);
+			if(stream->legacy_rid) {
+				a = janus_sdp_attribute_create("simulcast", " recv rid=%s", rids);
+			} else {
+				a = janus_sdp_attribute_create("simulcast", " recv %s", rids);
+			}
 			m->attributes = g_list_append(m->attributes, a);
 		}
 		if(!janus_ice_is_full_trickle_enabled()) {
