@@ -4,7 +4,7 @@
  * \brief  Janus Unix Sockets transport plugin
  * \details  This is an implementation of a Unix Sockets transport for the
  * Janus API. This means that, with the help of this module, local
- * applications can use Unix Sockets to make requests to the gateway.
+ * applications can use Unix Sockets to make requests to Janus.
  * This plugin can make use of either the \c SOCK_SEQPACKET or the
  * \c SOCK_DGRAM socket type according to what you configure, so make
  * sure you're using the right one when writing a client application.
@@ -209,19 +209,27 @@ int janus_pfunix_init(janus_transport_callbacks *callback, const char *config_pa
 		return -1;
 	}
 
-	/* This is the callback we'll need to invoke to contact the gateway */
+	/* This is the callback we'll need to invoke to contact the Janus core */
 	gateway = callback;
 
 	/* Read configuration */
 	char filename[255];
-	g_snprintf(filename, 255, "%s/%s.cfg", config_path, JANUS_PFUNIX_PACKAGE);
+	g_snprintf(filename, 255, "%s/%s.jcfg", config_path, JANUS_PFUNIX_PACKAGE);
 	JANUS_LOG(LOG_VERB, "Configuration file: %s\n", filename);
 	janus_config *config = janus_config_parse(filename);
+	if(config == NULL) {
+		JANUS_LOG(LOG_WARN, "Couldn't find .jcfg configuration file (%s), trying .cfg\n", JANUS_PFUNIX_PACKAGE);
+		g_snprintf(filename, 255, "%s/%s.cfg", config_path, JANUS_PFUNIX_PACKAGE);
+		JANUS_LOG(LOG_VERB, "Configuration file: %s\n", filename);
+		config = janus_config_parse(filename);
+	}
 	if(config != NULL) {
 		/* Handle configuration */
 		janus_config_print(config);
+		janus_config_category *config_general = janus_config_get_create(config, NULL, janus_config_type_category, "general");
+		janus_config_category *config_admin = janus_config_get_create(config, NULL, janus_config_type_category, "admin");
 
-		janus_config_item *item = janus_config_get_item_drilldown(config, "general", "json");
+		janus_config_item *item = janus_config_get(config, config_general, janus_config_type_item, "json");
 		if(item && item->value) {
 			/* Check how we need to format/serialize the JSON output */
 			if(!strcasecmp(item->value, "indented")) {
@@ -240,7 +248,7 @@ int janus_pfunix_init(janus_transport_callbacks *callback, const char *config_pa
 		}
 
 		/* Check if we need to send events to handlers */
-		janus_config_item *events = janus_config_get_item_drilldown(config, "general", "events");
+		janus_config_item *events = janus_config_get(config, config_general, janus_config_type_item, "events");
 		if(events != NULL && events->value != NULL)
 			notify_events = janus_is_true(events->value);
 		if(!notify_events && callback->events_is_enabled()) {
@@ -254,13 +262,13 @@ int janus_pfunix_init(janus_transport_callbacks *callback, const char *config_pa
 		}
 
 		/* Setup the Janus API Unix Sockets server(s) */
-		item = janus_config_get_item_drilldown(config, "general", "enabled");
+		item = janus_config_get(config, config_general, janus_config_type_item, "enabled");
 		if(!item || !item->value || !janus_is_true(item->value)) {
 			JANUS_LOG(LOG_WARN, "Unix Sockets server disabled (Janus API)\n");
 		} else {
-			item = janus_config_get_item_drilldown(config, "general", "path");
+			item = janus_config_get(config, config_general, janus_config_type_item, "path");
 			char *pfname = (char *)(item && item->value ? item->value : NULL);
-			item = janus_config_get_item_drilldown(config, "general", "type");
+			item = janus_config_get(config, config_general, janus_config_type_item, "type");
 			const char *type = item && item->value ? item->value : "SOCK_SEQPACKET";
 			dgram = FALSE;
 			if(!strcasecmp(type, "SOCK_SEQPACKET")) {
@@ -285,13 +293,13 @@ int janus_pfunix_init(janus_transport_callbacks *callback, const char *config_pa
 			}
 		}
 		/* Do the same for the Admin API, if enabled */
-		item = janus_config_get_item_drilldown(config, "admin", "admin_enabled");
+		item = janus_config_get(config, config_admin, janus_config_type_item, "admin_enabled");
 		if(!item || !item->value || !janus_is_true(item->value)) {
 			JANUS_LOG(LOG_WARN, "Unix Sockets server disabled (Admin API)\n");
 		} else {
-			item = janus_config_get_item_drilldown(config, "admin", "admin_path");
+			item = janus_config_get(config, config_admin, janus_config_type_item, "admin_path");
 			char *pfname = (char *)(item && item->value ? item->value : NULL);
-			item = janus_config_get_item_drilldown(config, "admin", "admin_type");
+			item = janus_config_get(config, config_admin, janus_config_type_item, "admin_type");
 			const char *type = item && item->value ? item->value : "SOCK_SEQPACKET";
 			if(!strcasecmp(type, "SOCK_SEQPACKET")) {
 				admin_dgram = FALSE;
